@@ -1,21 +1,25 @@
 package OpenInteract;
 
-# $Id: OpenInteract.pm,v 1.1 2001/07/11 12:33:03 lachoy Exp $
+# $Id: OpenInteract.pm,v 1.7 2001/08/13 05:10:47 lachoy Exp $
 
 use strict;
 use Apache::Constants qw( :common :remotehost );
 use Data::Dumper      qw( Dumper );
 
 @OpenInteract::ISA     = ();
-$OpenInteract::VERSION = sprintf("%d.%02d", q$Revision: 1.1 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
 
 # Generic separator used in display
 
 my $SEP = '=' x 30;
 
+# Keep track of what's been require'd
+
+my %REQ = ();
+
 
 sub handler {
-    my $apache = shift;
+    my ( $apache ) = @_;
 
     # Create the big cheese object (aka, "Big R") and populate with some
     # basic info
@@ -25,7 +29,8 @@ sub handler {
         _send_html( $apache, $@ );
         return OK;
     }
-    $R->DEBUG && $R->scrib( 1, "\n\n$SEP\nRequest started:", scalar localtime( $R->{time} ) );
+    $R->DEBUG && $R->scrib( 1, "\n\n$SEP\nRequest started:", scalar localtime( $R->{time} ), "\n",
+                               "path: (", $apache->parsed_uri->path, ") PID: ($$)" );
 
     # Go through all of our important steps -- we setup the basic
     # environment, generate the content and run additional routines after
@@ -38,19 +43,17 @@ sub handler {
     my ( $page );
     eval {
         _setup_apache( $R, $apache );
-        _connect_to_database( $R );
         _setup_cache( $R );
         _parse_uri( $R );
         _find_action_handler( $R );
         _setup_cookies_and_session( $R );
         _setup_authentication( $R );
-        _setup_context( $R );
         _setup_theme( $R );
         $page = _run_content_handler( $R );
         _finish_cookies_and_session( $R );
     }; 
     if ( $@ ) {
-        warn " --EXITED WITH ERROR from main handler eval block.\n";
+        warn " --EXITED WITH ERROR from main handler eval block\nError: $@\n";
         return _bail( $@ );
     }
 
@@ -76,8 +79,13 @@ sub _setup_request {
     # Read the stash class from our httpd.conf and grab the config
     # object
 
-    my $STASH_CLASS   = $apache->dir_config( 'StashClass' );
-    my $C             = $STASH_CLASS->get_stash( 'config' );
+    my $STASH_CLASS = $apache->dir_config( 'StashClass' );
+    unless ( $REQ{ $STASH_CLASS } ) {
+        eval "require $STASH_CLASS";
+        die "Cannot require stash class ($STASH_CLASS)!\n" if ( $@ );
+        $REQ{ $STASH_CLASS }++;
+    }
+    my $C = $STASH_CLASS->get_stash( 'config' );
     unless ( ref $C and scalar keys %{ $C } ) {
         die "Cannot find configuration object from stash class ($STASH_CLASS). Cannot continue!";
     }
@@ -86,10 +94,15 @@ sub _setup_request {
     # info
 
     my $REQUEST_CLASS = $C->{request_class};
+    unless ( $REQ{ $REQUEST_CLASS } ) {
+        eval "require $REQUEST_CLASS";
+        die "Cannot require request class ($REQUEST_CLASS)!\n" if ( $@ );
+        $REQ{ $REQUEST_CLASS }++;
+    }
     my $R = $REQUEST_CLASS->instance;
     $R->{stash_class} = $STASH_CLASS;
-  
-    $R->{time} = time;
+    $R->{pid}         = $$;
+    $R->{time}        = time;
     return $R;
 }
 
@@ -115,25 +128,6 @@ sub _setup_apache {
 }
 
 
-# Connect to database and store handle
-
-sub _connect_to_database {
-    my ( $R ) = @_;
-    my $db = eval { OpenInteract::DBI->connect( $R->CONFIG->{db_info} ) };
-    if ( $@ ) {
-        OpenInteract::Error->set( { system_msg => $@,
-                                    type       => 'db',
-                                    extra      => $R->CONFIG->{db_info} } );
-        $R->scrib( 0, "Cannot connect to database! Immediately exiting process!" );
-        my $error_msg = $R->throw( { code => 11 } );
-        _send_html( $R->apache, $error_msg, $R );
-        die OK . "\n";
-  }
-    $R->stash( 'db', $db );
-    return undef;
-}
-
-
 # Create the cache object if we're supposed to
 
 sub _setup_cache {
@@ -160,7 +154,9 @@ sub _setup_cache {
 sub _parse_uri {
     my ( $R ) = @_;
 
-    # Get the Apache::URI object and put it in $R
+    # Get the Apache::URI object and put it in $R 
+
+    # TODO: Do we EVER retrieve the URI object from the stash? Why put it there?
 
     my $apache = $R->apache;
     my $u = $apache->parsed_uri;
@@ -170,7 +166,14 @@ sub _parse_uri {
     # so we can shift items from one and still keep the original; we
     # also get the action name from the first item in the path
 
+    my $location = $apache->location;
     my $path = $u->path;
+    $R->DEBUG && $R->scrib( 1, "Original path: ($path)" );
+    if ( $location ne '/' ) {
+        $path =~ s/^$location//;
+        $R->{path}->{location} = $location;
+        $R->DEBUG && $R->scrib( 1, "Modified path by removing ($location): ($path)" );
+    }
     my @choices = split /\//, $path;
     shift @choices;
     $R->DEBUG && $R->scrib( 1, "Items in the path: ", join( " // ", @choices ) );
@@ -195,7 +198,7 @@ sub _parse_uri {
     $R->{path}->{original} .=  '?' . $u->query  if ( $u->query );
     $R->DEBUG && $R->scrib( 1, "Original path/query string set to: $R->{path}->{original}" );
     return undef;
-}   
+}
 
 
 # Match up the URL path to the UI action (Conductor) and store the
@@ -206,11 +209,11 @@ sub _find_action_handler {
     ( $R->{ui}->{class}, $R->{ui}->{method} ) = $R->lookup_conductor( $R->{ui}->{action} ); 
     unless ( $R->{ui}->{class} ) {
         $R->scrib( 0, " Conductor not found; displaying oops page." );
-        eval { $R->throw( { code       => 301,
-                            type       => 'file',
-                            user_msg   => "Bad URL",
-                            system_msg => "Cannot find conductor for $R->{ui}->{action}",
-                            extra      => { url => $R->{path}->{original} } } ) };
+        eval { $R->throw({ code       => 301,
+                           type       => 'file',
+                           user_msg   => "Bad URL",
+                           system_msg => "Cannot find conductor for $R->{ui}->{action}",
+                           extra      => { url => $R->{path}->{original} } }) };
         if ( $@ ) {
             _send_html( $R->apache, $@, $R );
             die OK . "\n";
@@ -223,11 +226,12 @@ sub _find_action_handler {
 
 sub _setup_cookies_and_session {
     my ( $R ) = @_;
+    warn "trying to setup cookies/session\n";
     eval {
-        $R->DEBUG && $R->scrib( 1, "Trying to use cookie class: ", $R->cookies );
+        $R->DEBUG && $R->scrib( 2, "Trying to use cookie class: ", $R->cookies );
         $R->cookies->parse;
         $R->DEBUG && $R->scrib( 2, "Cookies in:", Dumper( $R->{cookie}->{in} ) );
-        $R->DEBUG && $R->scrib( 1, "Trying to use session class: ", $R->session );
+        $R->DEBUG && $R->scrib( 2, "Trying to use session class: ", $R->session );
         $R->session->parse;
     };
     if ( $@ ) { 
@@ -285,16 +289,19 @@ sub _setup_authentication {
 sub _setup_theme {
     my ( $R ) = @_;
     my $C = $R->CONFIG;
-    $R->{theme} = ( $R->{auth}->{user} ) 
-                    ? eval { $R->{auth}->{user}->theme } 
+    $R->{theme} = ( $R->{auth}->{user} and $R->{auth}->{user}->{theme_id} )
+                    ? eval { $R->{auth}->{user}->theme }
                     : eval { $R->theme->fetch( $C->{default_objects}->{theme} ) };
     if ( $@ ) {
-        OpenInteract::Error->set( SPOPS::Error->get );
-        $R->throw( { code => 404 } );
-        $R->scrib( 0, "Error! Cannot retrieve theme! (Class: ", $R->theme, ") ",
-                      "Help! $OpenInteract::Error::system_msg" );
-        my $error_msg = qq|Fundamental part of OpenInteract (themes) not functioning; please contact | .
-                        qq|system administrator (<a href="mailto:$C->{admin_email}">$C->{admin_email}</a>).|;
+        my $ei = SPOPS::Error->get;
+        OpenInteract::Error->set( $ei );
+        $R->throw({ code => 404 });
+        $R->scrib( 0, "Error! Cannot retrieve theme! ( Class: ", $R->theme, ")",
+                      "with error ($@ / $ei->{system_msg}) Help!" );
+        my $error_msg = <<THEMERR;
+Fundamental part of OpenInteract (themes) not functioning; please contact the
+system administrator (<a href="mailto:$C->{admin_email}">$C->{admin_email}</a>).
+THEMERR
         _send_html( $R->apache, $error_msg, $R );
         die OK . "\n";
     }
@@ -361,27 +368,6 @@ sub _cleanup {
     return undef;
 }
 
-# Grab the group context; currently, it's in a GET/POST parameter
-# named 'gctxt' but this could change; what should not change is that
-# the key 'group_context' in $R holds this information.
-#
-# Note that this will probably change to be a behavior installed by
-# the 'group_community' package (it really shouldn't be here)
-
-sub _setup_context {
-    my ( $R ) = @_;
-    if ( $R->CONFIG->{context}->{group} ) {
-        my $context_id = $R->apache->param( 'gctxt' ) || $R->CONFIG->{default_objects}->{group};
-        $R->DEBUG && $R->scrib( 1, "Creating group context using ID ($context_id)" );
-        $R->{context}->{group} = eval { $R->group->fetch( $context_id ) };
-        if ( $@ ) {
-            $R->scrib( 0, "Cannot create group (", $R->group, ":: $context_id) for context!",
-                          "Error: $@\n", Dumper( SPOPS::Error->get ) );
-        }
-    }
-    return undef;
-}
-
 1;
 
 __END__
@@ -420,7 +406,7 @@ Create/retrieve the request object.
 
 =item *
 
-Create the Apache::Request object
+Create the C<Apache::Request> object
 
 =item *
 
@@ -428,10 +414,6 @@ Create the config (if it is not already around)
 
 =item *
 
-Create the db handle (if it is not already around)
-
-=item *
-
 Create the cache object if we are supposed to use it
 
 =item *
