@@ -1,6 +1,6 @@
 package OpenInteract::ApacheStartup;
 
-# $Id: ApacheStartup.pm,v 1.9 2001/07/11 12:26:27 lachoy Exp $
+# $Id: ApacheStartup.pm,v 1.13 2001/08/18 19:14:22 lachoy Exp $
 
 use strict;
 use OpenInteract::Startup;
@@ -8,7 +8,7 @@ use OpenInteract::Startup;
 use constant DEBUG => 0;
 
 $OpenInteract::ApacheStartup::VERSION   = '1.07';
-$OpenInteract::ApacheStartup::Revision  = substr(q$Revision: 1.9 $, 10);
+$OpenInteract::ApacheStartup::Revision  = substr(q$Revision: 1.13 $, 10);
 
 # Create a handler to put the X-Forwarded-For header into the IP
 # address -- thanks Stas! (perl.apache.org/guide/)
@@ -32,8 +32,8 @@ sub initialize {
 
     # We read the base config in first, so we can snag the apache modules
 
-    my $BASE_CONFIG = OpenInteract::Startup->read_base_config( { filename => $bc_file } );
-    die "Cannot create base configuration from ($bc_file)!" unless ( $BASE_CONFIG );
+    my $BASE_CONFIG = OpenInteract::Startup->read_base_config({ filename => $bc_file });
+    unless ( $BASE_CONFIG ) { die "Cannot create base configuration from ($bc_file)!"; }
     DEBUG && _w( 1, " --base configuration read in ok." );
 
     # Read in all the Apache classes -- do this separately since we need
@@ -46,7 +46,7 @@ sub initialize {
     # 'PerlModule Apache::DBI' before the PerlRequire statement into the
     # modperl httpd.conf
 
-    my $config_dir = join( '/', $BASE_CONFIG->{website_dir}, 
+    my $config_dir = join( '/', $BASE_CONFIG->{website_dir},
                                 $BASE_CONFIG->{config_dir} );
     OpenInteract::Startup->require_module({ filename => "$config_dir/apache.dat" });
     DEBUG && _w( 1, " --apache modules read in ok." );
@@ -63,19 +63,25 @@ sub initialize {
     DEBUG && _w( 1, " --main initialization completed ok." );
 
     OpenInteract::Startup->require_module({ class => $C->{session_info}->{class} });
+    DEBUG && _w( 1, " -- require the session class ok" );
 
     # Figure out how to do this more cleanly in the near future -- maybe
     # just do it by hand for this special class?
 
     push @{ $init_class }, 'OpenInteract::PackageRepository';
 
-    # Stas Beckman (stas@stason.org) wrote up a section in the
-    # mod_perl developer guide (perl.apache.org/guide/) about how this
-    # 'install_driver' thing saves memory. <shrug>
+    # Initialize the DBI drivers. Stas Beckman (stas@stason.org) wrote
+    # up a section in the mod_perl developer guide
+    # (perl.apache.org/guide/) about how this 'DBI->install_driver()' thing
+    # saves memory.
 
     my $db_info = $C->{db_info};
-    DBI->install_driver( $db_info->{driver_name} );
-    DEBUG && _w( 1, " --installed DBD driver ($db_info->{driver_name}) ok." );
+    my %dbd_installed = ();
+    foreach my $db_key ( keys %{ $db_info } ) {
+        next if ( $dbd_installed{ $db_info->{ $db_key }{driver_name} } );
+        DBI->install_driver( $db_info->{ $db_key }{driver_name} );
+        DEBUG && _w( 1, " --installed DBD driver ($db_info->{ $db_key }{driver_name}) ok." );
+    }
 
     # Check to see if the proxy subroutine has been loaded; if not, create it
 
@@ -107,18 +113,14 @@ sub initialize {
     if ( $OS=~/Win/i ) {
         DEBUG && _w( 1, "Running initialization for Windows." );
         srand;
-        $cache_class->class_initialize(     { config => $C } )  if ( $cache_info->{use} );
-        $ipc_cache_class->class_initialize( { config => $C } )  if ( $cache_info->{use_ipc} );
+        $cache_class->class_initialize({ config => $C })     if ( $cache_info->{use} );
+        $ipc_cache_class->class_initialize({ config => $C }) if ( $cache_info->{use_ipc} );
 
         my $REQUEST_CLASS = $BASE_CONFIG->{request_class};
         $REQUEST_CLASS->setup_aliases;
 
         OpenInteract::Startup->initialize_spops({ config => $C, class => $init_class });
 
-        eval { OpenInteract::Template::Toolkit->initialize( { config => $C } ) };
-        my $tmpl_status = ( $@ ) ? $@ : 'ok';
-        DEBUG && _w( 1, sprintf( "%-40s: %-30s","init: Template Toolkit", $tmpl_status ) );
-
         eval { OpenInteract::Error::Main->initialize({ config => $C }) };
         my $err_status  = ( $@ ) ? $@ : 'ok';
         DEBUG && _w( 1, sprintf( "%-40s: %-30s","init: Error Dispatcher", $err_status ) );
@@ -142,12 +144,22 @@ sub initialize {
             # received (probably not necessary using mysql, but for heavier
             # databases it can be a Good Thing)
 
-            OpenInteract::DBI->connect( $db_info );
+            # TODO: See if this is the reason why we get the 'not
+            # creating $R' problems discussed briefly at the bottom of
+            # OpenInteract.pm
 
-            $cache_class->class_initialize(     { config => $C })  if ( $cache_info->{use} );
-            $ipc_cache_class->class_initialize( { config => $C })  if ( $cache_info->{use_ipc} );
+            foreach my $db_key ( keys %{ $db_info } ) {
+                eval { OpenInteract::DBI->connect( $db_info->{ $db_key } ) };
+                if ( $@ ) {
+                    warn "Error trying to run first connect for database ",
+                         "connection key ($db_key): $@\n";
+                }
+            }
 
-            # Tell OpenInteract::Request to setup aliases if they haven't already
+            $cache_class->class_initialize({ config => $C })      if ( $cache_info->{use} );
+            $ipc_cache_class->class_initialize({ config => $C })  if ( $cache_info->{use_ipc} );
+
+            # Tell OpenInteract::Request to setup aliases if it hasn't already
 
             my $REQUEST_CLASS = $BASE_CONFIG->{request_class};
             $REQUEST_CLASS->setup_aliases;
@@ -157,18 +169,12 @@ sub initialize {
             OpenInteract::Startup->initialize_spops({ config => $C, 
                                                       class  => $init_class });
 
-            # Create the persistent template object for our website
-
-            eval { OpenInteract::Template::Toolkit->initialize({ config => $C }) };
-            my $tmpl_status = ( $@ ) ? $@ : 'ok';
-            DEBUG && _w( 0, sprintf( "%-40s: %-30s","init: Template Toolkit", $tmpl_status ) );
-
             # Create a list of error handlers for our website
 
             eval { OpenInteract::Error::Main->initialize({ config => $C }) };
             my $err_status  = ( $@ ) ? $@ : 'ok';
             DEBUG && _w( 0, sprintf( "%-40s: %-30s","init: Error Dispatcher", $err_status ) );
-        }; 
+        };
         Apache->push_handlers( PerlChildInitHandler => $init_sub );
     }
 
