@@ -1,6 +1,6 @@
 package OpenInteract::Handler::BasicPage;
 
-# $Id: BasicPage.pm,v 1.12 2001/07/13 15:23:27 lachoy Exp $
+# $Id: BasicPage.pm,v 1.14 2001/08/22 04:21:19 lachoy Exp $
 
 use strict;
 use SPOPS::Secure  qw( :level );
@@ -9,15 +9,15 @@ use Data::Dumper   qw( Dumper );
 use File::Basename qw();
 
 @OpenInteract::Handler::BasicPage::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure  );
-$OpenInteract::Handler::BasicPage::VERSION = sprintf("%d.%02d", q$Revision: 1.12 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::BasicPage::VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::BasicPage::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::BasicPage::default_method    = 'listing';
 @OpenInteract::Handler::BasicPage::forbidden_methods = ();
-%OpenInteract::Handler::BasicPage::security          = ( 
-   listing => SEC_LEVEL_WRITE, show   => SEC_LEVEL_NONE, 
+%OpenInteract::Handler::BasicPage::security          = (
+   listing => SEC_LEVEL_WRITE, show   => SEC_LEVEL_NONE,
    edit    => SEC_LEVEL_WRITE, remove => SEC_LEVEL_WRITE,
-   notify  => SEC_LEVEL_READ 
+   notify  => SEC_LEVEL_READ
 );
 
 
@@ -47,9 +47,9 @@ my $DEFAULT_EXPIRE = 60 * 60 * 24 * 7 * 52;
 
 sub _get_task {
     my ( $class ) = @_;
-    my $R = OpenInteract::Request->instance; 
+    my $R = OpenInteract::Request->instance;
     return 'show'  if ( $R->{path}->{full}->[0] ne 'BasicPage' );
-    return lc shift @{ $R->{path}->{current} } 
+    return lc shift @{ $R->{path}->{current} }
                 || $OpenInteract::Handler::BasicPage::default_method;
 }
 
@@ -63,8 +63,8 @@ sub listing {
     my $R = OpenInteract::Request->instance;
     my $selected_dir = $R->apache->param( 'selected_dir' );
     my $select_all = ( $selected_dir eq 'EXPANDALL' );
-    my $params = { main_script  => MAIN_SCRIPT, 
-                   selected_dir => $selected_dir, 
+    my $params = { main_script  => MAIN_SCRIPT,
+                   selected_dir => $selected_dir,
                    error_msg    => $p->{error_msg} };
     $params->{dir_list} = eval { $R->static_page->list_directories() };
     if ( $@ ) {
@@ -77,15 +77,15 @@ sub listing {
     # listref of files it contains
 
     if ( $selected_dir ) {
-        $params->{children_files} = $R->static_page->fetch_iterator({ 
+        $params->{children_files} = $R->static_page->fetch_iterator({
                                                column_group => 'listing',
                                                where        => 'directory = ?',
                                                value        => [ $selected_dir ] });
     }
 
     $R->{page}->{title} = 'Listing of Documents';
-    return $R->template->handler( {}, $params, 
-                                  { db      => 'static_dir_listing', 
+    return $R->template->handler( {}, $params,
+                                  { db      => 'static_dir_listing',
                                     package => 'static_page' } );
 }
 
@@ -93,11 +93,11 @@ sub listing {
 sub show {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { main_script => MAIN_SCRIPT, 
+    my $params = { main_script => MAIN_SCRIPT,
                    error_msg   => $p->{error_msg} };
 
     my $page    = $p->{static_page};
-    my $do_edit = ( $R->apache->param( 'edit' ) and 
+    my $do_edit = ( $R->apache->param( 'edit' ) and
                     $p->{level} >= SEC_LEVEL_WRITE );
 
     # If we are not passed a page, we need to try and grab the
@@ -105,9 +105,9 @@ sub show {
     # create the page, we return an error message unless we've said we
     # want to edit it.
 
-    my $location = $p->{location} || 
-                   $R->apache->param( 'location' ) || 
-                   $R->{path}->{original}; 
+    my $location = $p->{location} ||
+                   $R->apache->param( 'location' ) ||
+                   $R->{path}->{original};
 
     my $storage = $R->CONFIG->{action}->{basicpage}->{storage};
 
@@ -186,8 +186,8 @@ sub show {
 
     if ( $do_edit ) {
         $page ||= $R->static_page->new;
-        $params->{static_page} = $page;   
-        $text_params = { db      => 'static_page_form', 
+        $params->{static_page} = $page;  
+        $text_params = { db      => 'static_page_form',
                          package => 'static_page' };
         $R->{page}->{title}   = 'Edit a Document';
         my $update_items = eval { $page->fetch_updates( 5 ) } || [];
@@ -198,13 +198,13 @@ sub show {
         }
     }
     else {
-    
+
         # Ensure the page is viewable right now
-    
+
         unless ( my $is_active = $class->_check_active_date( $page_info ) ) {
             $R->DEBUG && $R->scrib( 1, "Page has been marked as non-active ($is_active); returning error message." );
             $R->{page}->{title} = 'Page not yet active';
-            return '<h2 align="center">Not Active</h2>' . 
+            return '<h2 align="center">Not Active</h2>' .
                    '<p>Sorry, this page is not yet active.</p>';
         }
 
@@ -216,7 +216,7 @@ sub show {
         # the default
 
         $R->{page}->{_template_name_} = $page_info->{main_template};
-    
+
         # You can split your page into multiple viewable pages -- see
         # _split_pages() for more info
 
@@ -244,7 +244,7 @@ sub edit {
     # then this is a new page
 
     my $is_new = ( ! $location );
-  
+ 
     $R->DEBUG && $R->scrib( 1, "Trying to modify location <<$location>>" );
     my $page = $p->{page};
     if ( $location ) {
@@ -263,7 +263,7 @@ sub edit {
     my $obj_level = ( $page ) ? $page->{tmp_security_level} : SEC_LEVEL_WRITE;
     if ( $obj_level < SEC_LEVEL_WRITE ) {
         my $user_msg = 'Sorry, you do not have access to modify this document. Returning to listing.';
-        return $class->show( { page => $page, error_msg => $user_msg } );   
+        return $class->show( { page => $page, error_msg => $user_msg } );  
     }
 
     # Again, create a new object if one wasn't created before.
@@ -275,7 +275,7 @@ sub edit {
         $R->DEBUG && $R->scrib( 1, "Find value for field <<$field>>: ", $apr->param( $field ) );
         $page->{ $field } = $apr->param( $field );
     }
-  
+ 
     $page->{active_on}  = $class->date_read( 'active_on' ) || $page->now();
     $page->{expires_on} = $class->date_read( 'expires_on' );
     unless ( $page->{expires_on} ) {
@@ -285,15 +285,15 @@ sub edit {
         my @time_info = localtime( $expire_time );
         $page->{expires_on} = join( '-', $time_info[5] + 1900, $time_info[4] + 1, $time_info[3] );
     }
-  
+ 
     # If the user changed the location, be sure to send the
     # old ID value to the update
-  
+ 
     my $update_id = undef;
     unless ( $is_new or $page->{location} eq $location ) {
         $update_id = $location;
     }
-  
+ 
     $R->DEBUG && $R->scrib( 1, "Contents of Page object before saving:\n", Dumper( $page ) );
     eval { $page->save( { is_add => $is_new, use_id => $update_id } ) };
     if ( $@ ) {
@@ -304,10 +304,10 @@ sub edit {
                'Cannot save document object! See error log for details.'
            }
 
-    $R->DEBUG && $R->scrib( 1, "Document saved ok (New location: $page->{location});", 
+    $R->DEBUG && $R->scrib( 1, "Document saved ok (New location: $page->{location});",
                                "returning to view of document." );
     $p->{static_page} = $page;
-    return $class->show( $p ); 
+    return $class->show( $p );
 }
 
 
@@ -321,7 +321,7 @@ sub remove {
         }
         if ( $page->{tmp_security_level} < SEC_LEVEL_WRITE ) {
             my $user_msg = 'Sorry, you do not have access to remove this document. Returning to listing.';
-            return $class->listing( { error_msg => $user_msg } );   
+            return $class->listing( { error_msg => $user_msg } );  
         }
 
         eval { $page->remove };
@@ -339,13 +339,13 @@ sub _retrieve_object {
     my ( $class, $location ) = @_;
     my $R = OpenInteract::Request->instance;
     $R->DEBUG && $R->scrib( 1, "Trying to retrieve OBJECT with location ($location)" );
-  
+ 
     # Chop off any query strings and put in a case-consistent format
-  
+ 
     $location    = $class->_remove_query_string( $location );
     $location    = lc $location;
     my $home_name = $R->CONFIG->{action}->{basicpage}->{object_home_name};
-  
+ 
     my @locations = ( $location );
     if ( $location =~ m|/$| ) {
         $R->DEBUG && $R->scrib( 1, "Trying to grab a directory; adding $home_name" );
@@ -406,7 +406,7 @@ sub _retrieve_file {
     my $page_info = $class->_fetch_file( $location, "$location/$home_name" );
     return undef unless ( $page_info );
     $page_info->{_storage} = 'filesystem';
-    return $page_info; 
+    return $page_info;
 }
 
 
@@ -420,7 +420,7 @@ sub _fetch_file {
 
     my $html_dir = $R->CONFIG->get_dir( 'html' );
     $html_dir =~ s|/$||;
-  
+ 
     foreach my $location ( @loc ) {
 
         # Security -- remove all '.' from the beginning of the location
@@ -430,9 +430,9 @@ sub _fetch_file {
         # In the future we might flag these as bad requests and simply
         # bail with a stern warning.
 
-        $location =~ s/^\.+//; 
+        $location =~ s/^\.+//;
         $location =~ s/\.\./_/;
-    
+
         my $full_location = join( '', $html_dir, $location );
         next unless ( -f $full_location );
         $R->DEBUG && $R->scrib( 1, "Filesystem location ($full_location) ",
@@ -446,11 +446,10 @@ sub _fetch_file {
         if ( $R->basicpage and $R->basicpage->isa( 'SPOPS::Secure' ) ) {
             $R->DEBUG && $R->scrib( 1, "StaticPage object is protected by security. ",
                                        "Check security in filesystem." );
-            $level = eval { SPOPS::Secure::Hierarchy->check_security({
+            $level = eval { $R->basicpage->check_security({
                                  security_object_class => $R->security_object,
-                                 class                 => $R->basicpage, 
                                  object_id             => $location,
-		                         user                  => $R->{auth}->{user}, 
+		                         user                  => $R->{auth}->{user},
                                  group                 => $R->{auth}->{group},
                                  hierarchy_separator   => '/' }) };
             $R->scrib( 0, "Error found checking security: $@" )  if ( $@ );
@@ -474,7 +473,7 @@ sub _fetch_file {
 
         $R->DEBUG && $R->scrib( 1, "File requested can be displayed in the browser. ",
                                    "Reading in the file..." );
-    
+
         open( STATIC, $full_location ) || die "access";
         local $/ = undef;
         $page_info->{pagetext} = <STATIC>;
@@ -504,7 +503,7 @@ sub _fetch_file {
         # between them
 
         $page_info->{pagetext} =~ s|<body>(.*)?</body>|$1|i;
-    
+
         $page_info->{location} = $location;
         $page_info->{_filesystem} = $full_location;
         $R->DEBUG && $R->scrib( 2, "Page information being returned: ", Dumper( $page_info ) );
@@ -546,7 +545,7 @@ sub _split_pages {
         my $current_pagenum = $page_num;
         $this_page .= <<PCOUNT;
      <p align="right"><font size="-1">
-     [% comp( 'page_count', total_pages     = $total_pages, 
+     [% comp( 'page_count', total_pages     = $total_pages,
                             url             = '$page_info->{location}',
                             current_pagenum = $current_pagenum ) %]
      </font></p>
@@ -563,15 +562,17 @@ sub _add_object_boxes {
 
   # Do the related items
 #   push @{ $R->{boxes} }, { name => 'relateditems', params => { object => $page } };
- 
+
   # If this page has specified any boxes, push them onto the stack
   # unless they start with a '-', in which case set that name aside
   # so we can go through all of the boxes and remove it.
-  
+ 
     my %box_remove = ();
     if ( $page_info->{boxes} ) {
         my @boxes = split /\s+/, $page_info->{boxes};
         foreach my $box_name ( @boxes ) {
+            next if ( $box_name =~ /^\s*$/ );
+            $R->scrib( 1, "Adding box name ($box_name) from page definition" );
             if ( $box_name =~ s/^\-// ) {
                 $box_remove{ $box_name }++;
             }
@@ -579,7 +580,10 @@ sub _add_object_boxes {
                 push @{ $R->{boxes} }, $box_name;
             }
         }
-    
+
+        # Go through ALL the boxes and do a removal if necessary
+        # (TODO: we should move this processing to base_box)
+
         my @box_keep = ();
         foreach my $box_info ( @{ $R->{boxes} } ) {
             next if ( ref $box_info and $box_remove{ $box_info->{name} } );
@@ -592,9 +596,11 @@ sub _add_object_boxes {
     # If this person has WRITE access to the module, give them a box
     # so they can edit/remove this document
 
-    if ( $p->{level} >= SEC_LEVEL_WRITE ) { 
-        push @{ $R->{boxes} }, { name => 'edit_document_box', params => { location => $page_info->{location} } }, 
-                               { name => 'objectmodbox',      params => { object => $page_info->{_object} } };
+    if ( $p->{level} >= SEC_LEVEL_WRITE ) {
+        push @{ $R->{boxes} }, { name   => 'edit_document_box',
+                                 params => { location => $page_info->{location} } },
+                               { name   => 'object_modify_box',
+                                 params => { object => $page_info->{_object} } };
     }
     return undef;
 }
@@ -615,7 +621,7 @@ sub _check_active_date {
         $R->scrib( 0, "Problem while trying to evaluate days! Error: $@" );
         return 1;
     }
-  
+ 
     $R->DEBUG && $R->scrib( 1, "Active on: $page_info->{active_on}; Expires on: $page_info->{expires_on}\n",
                                "Active: $active_days; Now: $now_days; Expire: $expire_days" );
     return undef if ( $now_days < $active_days );
