@@ -1,6 +1,6 @@
 package OpenInteract::Handler::NewUser;
 
-# $Id: NewUser.pm,v 1.2 2001/03/13 12:20:18 lachoy Exp $
+# $Id: NewUser.pm,v 1.3 2001/07/09 14:23:30 lachoy Exp $
 
 use strict;
 use Email::Valid;
@@ -8,7 +8,7 @@ use Data::Dumper  qw( Dumper );
 use SPOPS::Secure qw( :level :scope );
 
 @OpenInteract::Handler::NewUser::ISA     = qw( OpenInteract::Handler::GenericDispatcher );
-$OpenInteract::Handler::NewUser::VERSION = sprintf("%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::NewUser::VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::NewUser::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::NewUser::default_method    = 'show';
@@ -70,7 +70,7 @@ sub edit {
  my $password            = $R->user->generate_random_code( 12, 'mixed' );
  $new_user->{password}   = $R->user->crypt_it( $password );
  $new_user->{removal_date} = $R->user->now( { time => $R->{time} + $REMOVAL_TIME } );
- my $new_uid = eval { $new_user->save };
+ eval { $new_user->save };
  
  # If there was an error, log it and send to the admin (error handler)
  if ( $@ ) {
@@ -85,7 +85,7 @@ sub edit {
 
  # Ensure that the user can read/write his/her own record!
  eval { $R->user->set_item_security({ class     => $R->user, 
-                                      object_id => $new_uid,
+                                      object_id => $new_user->{user_id},
                                       scope     => SEC_SCOPE_USER, 
                                       scope_id  => $new_user->{user_id},
                                       level     => SEC_LEVEL_WRITE }) };
@@ -98,7 +98,7 @@ sub edit {
 
  # ...otherwise, mark the user as the creator of his/her own record
  else {
-   $new_user->log_action_enter( 'create', $new_uid, $new_uid );
+   $new_user->log_action_enter( 'create', $new_user->{user_id}, $new_user->{user_id} );
  }
 
  # If that worked ok, send the user an email with the password created
