@@ -1,11 +1,11 @@
 package OpenInteract::Handler::Redirect;
 
-# $Id: Redirect.pm,v 1.4 2001/07/13 14:37:04 lachoy Exp $
+# $Id: Redirect.pm,v 1.5 2001/08/20 14:52:58 lachoy Exp $
 
 use strict;
 
 @OpenInteract::Redirect::ISA     = ();
-$OpenInteract::Redirect::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Redirect::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
 
 use constant DEBUG  => 0;
 
@@ -26,7 +26,7 @@ sub handler {
     my ( $class, $p ) = @_;
     $p->{url} ||= '/';
     my $R = OpenInteract::Request->instance;
-    $R->{page}->{_no_tmpl_}++;
+    $R->{page}->{_no_template_}++;
 
     # the url in the meta refresh tag doesn't like semicolons 
     $p->{url} =~ s/;/&/g; 
