@@ -213,10 +213,11 @@ sub edit {
         $R->DEBUG && $R->scrib( 1, "Trying level ($level) for ($object_class)",
                                    "($object_id) in scope ($info->[0]) ($info->[1])" );
         my $security_params = { security_object_class => $security_object_class,
-                                class     => $object_class,
-                                object_id => $object_id,
-                                scope     => $info->[0],
-                                scope_id  => $info->[1] };
+                                class          => $object_class,
+                                object_id      => $object_id,
+                                scope          => $info->[0],
+                                scope_id       => $info->[1],
+                                security_level => $level };
         my $method = ( $level eq 'clear' ) 
                        ? 'remove_item_security' : 'set_item_security';
         $success += eval { $SECURE_CLASS->$method( $security_params ) };
