@@ -1,18 +1,18 @@
 package OpenInteract::Handler::SystemBoxes;
 
-# $Id: SystemBoxes.pm,v 1.5 2001/07/13 14:41:11 lachoy Exp $
+# $Id: SystemBoxes.pm,v 1.8 2001/08/12 15:40:32 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
 
 @OpenInteract::Handler::SystemBoxes::ISA     = qw();
-$OpenInteract::Handler::SystemBoxes::VERSION = sprintf("%d.%02d", q$Revision: 1.5 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::SystemBoxes::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::SystemBoxes::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::SystemBoxes::default_method    = 'handler';
 @OpenInteract::Handler::SystemBoxes::forbidden_methods = ();
 %OpenInteract::Handler::SystemBoxes::security = (
- handler        => SEC_LEVEL_NONE, 
+ handler        => SEC_LEVEL_NONE,
  templates_used => SEC_LEVEL_READ
 );
 
@@ -20,7 +20,7 @@ my @BOX_KEYS = qw( name weight title template package class method code );
 
 my %USER_BOX_DEFAULTS = (
        template => 'user_info_box',
-       package  => 'base_box',    
+       package  => 'base_box',
        weight   => 1,
        title    => 'User Info',
        name     => 'user_info_box' );
@@ -35,14 +35,14 @@ my %ADMIN_BOX_DEFAULTS = (
 my %TEMPLATES_USED_BOX_DEFAULTS = (
        class    => 'OpenInteract::Handler::SystemBoxes',
        method   => 'templates_used',
-       weight   => 11,    
+       weight   => 11,
        title    => 'Templates Used',
        name     => 'templates_used_box' );
 
 my %LOGIN_BOX_DEFAULTS = (
        template => 'login_box',
        package  => 'base_box',
-       weight   => 1,               
+       weight   => 1,
        title    => 'Login',
        name     => 'login_box' );
 
@@ -66,9 +66,10 @@ sub handler {
         # Now, if the user is a member of the site admin group, add the
         # 'Admin Tools' box and the 'Templates Used' box.
 
-        my $site_admin_id = $conf->{default_objects}->{site_admin_group};
+        my $site_admin_id = $conf->{default_objects}{site_admin_group};
+        my $supergroup_id = $conf->{default_objects}{supergroup};
         foreach my $group ( @{ $R->{auth}->{group} } ) {
-            next unless ( $group->{group_id} == $site_admin_id or $group->{group_id} == 1 );
+            next unless ( $group->{group_id} =~ /^($site_admin_id|$supergroup_id)$/ );
             $R->DEBUG && $R->scrib( 1, "User is in site admin group:  generating info for ",
                                        "admin_tools_box and templates used box" );
             push @{ $R->{boxes} }, $class->_assign_params({
@@ -77,6 +78,7 @@ sub handler {
             push @{ $R->{boxes} }, $class->_assign_params({
                                              action => 'templates_used_box', 
                                              defaults => \%TEMPLATES_USED_BOX_DEFAULTS });
+            last;
         }
         return undef;
     }
@@ -108,7 +110,8 @@ sub _assign_params {
 
 
 # Simple handler to put the list of templates used into a more
-# manageable form
+# manageable form -- note that we can handle template objects and
+# 'package::name' strings in \@{ $R->{templates_used} }
 
 sub templates_used {
     my ( $class ) = @_;
@@ -116,11 +119,21 @@ sub templates_used {
     my %tmpl_by_package = ();
     my %tmpl_used = ();
     foreach my $template ( @{ $R->{templates_used} } ) {
-        next if ( $tmpl_used{ $template->{package} }->{ $template->{name} } );
-        push @{ $tmpl_by_package{ $template->{package} } }, 
-                            { name => $template->{name}, 
-                              id   => $template->{template_id} };
-        $tmpl_used{ $template->{package} }->{ $template->{name} }++;
+        my ( $package, $name, $template_id, $full_name );
+        if ( ref $template ) {
+            ( $package, $name ) = ( $template->{package}, $template->{name} );
+            $template_id = $template->{template_id};
+        }
+        else {
+            ( $package, $name ) = $R->site_template->parse_name( $template );
+            $full_name = $template;
+        }
+        next if ( $tmpl_used{ $package }->{ $name } );
+        $tmpl_used{ $package }->{ $name }++;
+        push @{ $tmpl_by_package{ $package } }, 
+                            { name      => $name, 
+                              id        => $template_id,
+                              full_name => $full_name };
     }
     return $R->template->handler( {}, { templates_used => \%tmpl_by_package },
                                   { db      => 'templates_used_box', 
