@@ -1,16 +1,18 @@
 package OpenInteract::Handler::SystemDoc;
 
-# $Id: SystemDoc.pm,v 1.4 2001/07/13 15:32:00 lachoy Exp $
+# $Id: SystemDoc.pm,v 1.7 2001/08/24 21:00:23 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :level );
 use Data::Dumper  qw( Dumper );
-use Pod::Parser;
 use OpenInteract::Package;
 use OpenInteract::PackageRepository;
+use Pod::POM;
+
+my ( %POD_CACHE );
 
 @OpenInteract::Handler::SystemDoc::ISA     = qw( OpenInteract::Handler::GenericDispatcher SPOPS::Secure );
-$OpenInteract::Handler::SystemDoc::VERSION = sprintf("%d.%02d", q$Revision: 1.4 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::SystemDoc::VERSION = sprintf("%d.%02d", q$Revision: 1.7 $ =~ /(\d+)\.(\d+)/);
 
 $OpenInteract::Handler::SystemDoc::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::SystemDoc::default_method    = 'listing';
@@ -108,22 +110,22 @@ PACKAGE:
 sub module_list {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { error_msg   => $p->{error_msg}, 
+    my $params = { error_msg   => $p->{error_msg},
                    main_script => MAIN_SCRIPT  };
     my $view = $R->apache->param( 'view' ) || 'local';
 
     # Now sort INC and chop up the files into packages -- depending on
     # what option we were given, restrict the modules to either
     # SPOPS/OpenInteract or Everything
-  
+
     my %this_inc = %INC;
     my @top = ();
     my $website_name = $R->CONFIG->{website_name};
     $R->DEBUG && $R->scrib( 1, "Website name being filtered: ($website_name)" );
 
     if ( $view eq 'local' ) {
-        @top = ( [ 'OpenInteract', [ 'OpenInteract' ] ], 
-                 [ 'SPOPS', [ 'SPOPS' ] ], 
+        @top = ( [ 'OpenInteract', [ 'OpenInteract' ] ],
+                 [ 'SPOPS', [ 'SPOPS' ] ],
                  [ $website_name, [] ] );
         my @local_modules = sort grep /^(OpenInteract|SPOPS|$website_name)/, keys %this_inc;
         foreach my $local_module ( @local_modules ) {
@@ -195,8 +197,8 @@ sub show {
     my $package = $R->apache->param( 'package' );
     if ( $package ) {
         my $doc = $R->apache->param( 'doc' );
-        my $repos = OpenInteract::PackageRepository->fetch( 
-                                   undef, 
+        my $repos = OpenInteract::PackageRepository->fetch(
+                                   undef,
                                    { directory => $R->CONFIG->{dir}->{base} } );
         if ( $doc =~ /\.(html|txt|pod)$/ ) {
             my $full_filename = $repos->find_file( $package, "doc/$doc" );
@@ -208,25 +210,33 @@ sub show {
     }
     else {
         my $module = $p->{module} || $R->apache->param( 'module' );
-        $pod_file = `perldoc -l $module`; # ewww! ick! 
+
+        # ewww! ick!
+        $pod_file = $POD_CACHE{ $module } || `perldoc -l $module`;
         chomp $pod_file;
         $R->DEBUG && $R->scrib( 1, "Found ($pod_file) from module ($module)" );
         unless ( $pod_file ) {
-            $pod_file = $INC{ _uncolonify( $module ) };   
+            $pod_file = $INC{ _uncolonify( $module ) };
             $R->DEBUG && $R->scrib( 1, "Found ($pod_file) from module ($module)" );
         }
+        $POD_CACHE{ $module } = $pod_file  if ( -f $pod_file );
     }
 
     if ( -f $pod_file ) {
-        $R->DEBUG && $R->scrib( 1, "Trying to view pod in $pod_file" );
-        chdir( '/tmp' );
-        my $pod_text = `pod2html $pod_file`; # ewww! ick! 
-        $pod_text =~ s/^.*<BODY>//;
-        $pod_text =~ s|</BODY>.*$||;
-        return '<p>Sorry, no docs for this module.' unless ( $pod_text );
+        $R->DEBUG && $R->scrib( 1, "Trying to view pod in ($pod_file)" );
+        my $parser = Pod::POM->new();
+        my $pom = $parser->parse( $pod_file );
+        return '<p>Error parsing POD: ' . $parser->error() . '</p>' unless ( $pom );
+
+        eval { require OpenInteract::PodView };
+        return "<p>Error trying to load POD viewer: $@</p>"  if ( $@ );
+        my $doc = OpenInteract::PodView->print( $pom );
+        return '<p>Sorry, no docs for this module.</p>'      unless ( $doc );
+        $doc =~ s/^.*<BODY>//;
+        $doc =~ s|</BODY>.*$||;
         $R->{page}->{title} = 'OpenInteract System Documentation Display';
-        return $pod_text;
-    } 
+        return $doc;
+    }
 
     if ( -f $html_file ) {
         eval { open( HTML, $html_file ) || die $! };
@@ -309,14 +319,6 @@ B<Get more meta information>
 System documentation needs more meta information so we can better
 display title and other information on the listing page.
 
-B<Change POD parser>
-
-The C<pod2html> program is not entirely suitable for what we want to
-do. It would be nice to have code sections marked in a certain way and
-to handle internal references properly. (That is, something like:
-LE<lt>OpenInteract::PackageE<gt> would be handled with a link to the
-page within this documentation system.)
-
 =head1 BUGS
 
 None known.
