@@ -1,12 +1,12 @@
 package OpenInteract::Security;
 
-# $Id: Security.pm,v 1.8 2001/07/13 15:03:23 lachoy Exp $
+# $Id: Security.pm,v 1.10 2001/08/25 23:25:11 lachoy Exp $
 
 use strict;
 use SPOPS::Secure qw( :all );
 use Data::Dumper  qw( Dumper );
 
-$OpenInteract::Security::VERSION = sprintf("%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Security::VERSION = sprintf("%d.%02d", q$Revision: 1.10 $ =~ /(\d+)\.(\d+)/);
 
 
 # Pass in: 
@@ -23,7 +23,7 @@ sub fetch_by_object {
     my ( $class, $item, $p ) = @_;
     my $R = OpenInteract::Request->instance;
     my $object_id = $p->{oid} || $p->{object_id};
-    unless ( $item or ( $p->{class} and $object_id ) ) {
+    unless ( $item or ( $p->{class} and defined $object_id ) ) {
         my $msg = 'Cannot check security';
         my $system_msg = 'No item passed into method to check!';
         $R->scrib( 0, "Cannot retrieve security since no item passed in to check!" );
@@ -34,9 +34,9 @@ sub fetch_by_object {
         die $msg;
     }
 
-    my ( $obj_class, $oid ) = $class->_get_class_and_oid({ 
-                                          %{ $p }, 
-                                          object_id => $object_id, 
+    my ( $obj_class, $oid ) = $class->_get_class_and_oid({
+                                          %{ $p },
+                                          object_id => $object_id,
                                           item      => $item } );
 
     my $where = 'class = ? AND object_id = ? AND ( scope = ?';
