@@ -1,26 +1,66 @@
 package OpenInteract::SiteTemplate;
 
-# $Id: SiteTemplate.pm,v 1.3 2001/07/13 15:08:29 lachoy Exp $
+# $Id: SiteTemplate.pm,v 1.6 2001/07/30 12:16:20 lachoy Exp $
 
 use strict;
 use Data::Dumper qw( Dumper );
 
-$OpenInteract::SiteTemplate::VERSION = sprintf("%d.%02d", q$Revision: 1.3 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::SiteTemplate::VERSION = sprintf("%d.%02d", q$Revision: 1.6 $ =~ /(\d+)\.(\d+)/);
+
+my $NAME_SEPARATOR = '::';
+
+########################################
+# CLASS METHODS
+########################################
+
+# Parse a combined 'package::name' label into the package and
+# name. Yes, it's a simple format but we might need to change it in
+# the future, so why not create a method
+
+sub parse_name {
+    my ( $class, $full_name ) = @_;
+    return split /$NAME_SEPARATOR/, $full_name, 2;
+}
+
+
+# Create a combined 'package::name' label
+
+sub create_name {
+    my ( $item, $package, $name ) = @_;
+    if ( ref $item ) {
+        $package = $item->{package};
+        $name    = $item->{name};
+    }
+    return join $NAME_SEPARATOR, $package, $name;
+}
+
+# Retrieve a template by name. Note that $name can be a simple name or
+# the combined 'package::name'. We might also *require* that the name
+# is combined or that $package is passed in.
 
 sub fetch_by_name {
-    my ( $class, $name, $package ) = @_;
-    $name = lc $name;
+    my ( $class, $name, $package, $p ) = @_;
+    $name    = lc $name;
+    $package = lc $package;
+    $p     ||= {};
+    
     my $R = OpenInteract::Request->instance;
     $R->DEBUG && $R->scrib( 1, "Trying to retrieve template for name ($name) and ",
                                "package ($package)" );
 
+    my ( $cmb_package, $cmb_name ) = $class->parse_name( $name );
+    if ( $cmb_package and $cmb_name ) {
+        $package = $cmb_package;
+        $name    = $cmb_name;
+    }
+
     my $where = 'name = ?';
     my @value = ( $name );
     if ( $package ) {
         $where   .= ' AND package = ?';
         push @value, $package;
     }
-    my $obj = eval { $class->fetch_group({ where => $where, value => \@value }) };
+    my $obj = eval { $class->fetch_group({ %{ $p }, where => $where, value => \@value }) };
     if ( $@ ) {
         OpenInteract::Error->set( SPOPS::Error->get );
         $OpenInteract::Error::user_msg = "Cannot retrieve site template for name " .
@@ -36,6 +76,24 @@ sub fetch_by_name {
     return $obj->[0];
 }
 
+
+########################################
+# RULESET METHODS
+########################################
+
+sub ruleset_add {
+    my ( $class, $rs_table ) = @_;
+    push @{ $rs_table->{pre_save_action} }, \&calculate_update_time;
+    return __PACKAGE__;
+}
+
+
+sub calculate_update_time {
+    my ( $self, $p ) = @_;
+    $self->{last_update} = time;
+    return 1;
+}
+
 1;
 
 __END__
@@ -69,6 +127,11 @@ OpenInteract::SiteTemplate - Object to represent templates and accompanying info
     print "Template contents: $tmpl->{template}\n";
  }
 
+ # Parse the common 'package::name' format
+
+ my $full_name = 'base_box::main_box_shell';
+ my ( $tmpl_package, $tmpl_name ) = $R->site_template->parse_name( $full_name );
+
 =head1 DESCRIPTION
 
 SiteTemplate objects are used throughout OpenInteract -- in fact, on
@@ -87,6 +150,13 @@ based on the name and package specified.
 
 =head1 METHODS
 
+B<parse_name( $full_template_name )>
+
+Parse a full template name (in 'package::name' format) into the
+package and name comprising it.
+
+Returns a two-item list: C<( $package, $name )>.
+
 B<fetch_by_name( template_name, [ package_name ])>
 
 Retrieves template(s) by name and/or package. (Just from the database
@@ -100,13 +170,15 @@ Parameters:
 
 B<template_name> ($)
 
-Name of the template to retrieve.
+Name of the template to retrieve. Note that this can be the combined
+'package::name' format as well.
 
-B<package_name> ($)
+B<package_name> ($) (optional, see note)
 
 Name of the package to use for retrieving the template -- both the
 'template_name' and 'package_name' are required to uniquely identify a
-template.
+template. If you use the 'package::name' format for 'template_name'
+then this is optional, otherwise it is required.
 
 =head1 TO DO
 
