@@ -19,25 +19,7 @@ use OpenInteract::Startup;
 # properties before running this script.
 
 {
-    my $usage = "Usage: $0 --website_dir=/path/to/my_site";
-
-    # Get the options
-
-    my ( $OPT_website_dir );
-    GetOptions( 'website_dir=s' => \$OPT_website_dir );
-
-    if ( ! $OPT_website_dir and $ENV{OIWEBSITE} ) {
-        print "[db_to_files]: Using ($ENV{OIWEBSITE}) for 'website_dir'.\n";
-        $OPT_website_dir = $ENV{OIWEBSITE};
-    }
-
-    unless ( -d $OPT_website_dir ) {
-        die "$usage\n Parameter 'website_dir' must refer to an OpenInteract website directory!\n";
-    }
-
-    # Setup the OI environment
-
-    my $R = OpenInteract::Startup->setup_static_environment( $OPT_website_dir );
+    my $R = OpenInteract::Startup->setup_static_environment_options( $usage );
 
     # Find all the pages
 
